# Rudefox Burrow

Rudefox provides a pair of command line tools for the creation and management of Bitcoin wallets.

+ Use `burrow` on an air-gapped single-board computer, like a Raspberry Pi, to generate new seed words, obtain addresses and extended public keys from existing seed words and sign PSBT transactions offline.
+ Use `reynard` on a connected ("hot") device to view your balance and utxo's and to create new, unsigned transactions.

## Unique Feature: ShowMyWork

[**Rudefox ShowMyWork**](https://rudefox.io/blog/2020-07-16-show-my-work.html) is a feature of Burrow that enables non-technical end-users to audit the seed generation process in order to verify that the generated seed is derived directly from [8-sided dice](https://www.amazon.com/dp/B07XPMV4DQ/) rolls and a set of lookup-tables.  Read more [here](https://rudefox.io/blog/2020-07-16-show-my-work.html).

```bash
alice@cold.machine:~$ burrow mnemonic -i8 --bits 128

Input 11 x 8-sided dice rolls [1-8]: 12345678123

| 1 | 2 | 3 |  4   | 5 | 6 | 7 |  8   | 1 | 2 | 3 |
|000 001 010 01 | 1 100 101 110 1 | 11 000 001 010|
|  1. ahead     |  2. slight      |  3. scout     |

Input 11 x 8-sided dice rolls [1-8]: 12345678123

| 1 | 2 | 3 |  4   | 5 | 6 | 7 |  8   | 1 | 2 | 3 |
|000 001 010 01 | 1 100 101 110 1 | 11 000 001 010|
|  4. ahead     |  5. slight      |  6. scout     |

...

ahead slight scout ahead slight scout ahead slight scout ahead slight scan
```

## Menemonic Generation

```bash
alice@cold.machine:~$ burrow mnemonic --interactive --dice-entropy --bits 128
Input 50 dice rolls [1-6]: 234322343242422344161254151
Input 23 more dice rolls [1-6]: 33116265515343114314456
Mnemonic Sentence: mountain tilt wing silk rude fox almost volume wine media verify card
```

### Mnemonic Options

```bash
usage: burrow mnemonic
 -b,--bits <128|160|192|224|256>   bits of entropy (default: 256)
 -d,--dice-entropy                 use dice entropy source
 -e,--events <EVENT_STRING>        string representing events from entropy source
 -h,--help                         display help message
 -i,--interactive                  use interactive command line mode
```

## Xpub Export

```bash
alice@cold.machine:~$ burrow wallet --sentence "stove prefer lunch collect small orphan wasp size beyond auction 
guilt great" --passphrase apple
xpub6CKy5SECeJipZid8dF3bopoMGdRzd7hMJuPzMGesZCobrMSssZyASexzXuzRTPVLcqqdyAEZJKPMGvDthgZW2Z3mPHLohxEAVbkvGKAXjqx
```
### QR Code Generation

```bash-qrcode
alice@cold.machine:~$ burrow wallet --sentence "stove prefer lunch collect small orphan wasp size beyond auction 
guilt great" --passphrase apple -q

█▀▀▀▀▀█ ▄█   █▄█ ▀▄▀ ▀▄▄▄██▄  █▀▄ █▀▀▀▀▀█
█ ███ █ ▀  ▀▄█▄▄▄▄██▄▄▄█  █  █▄   █ ███ █
█ ▀▀▀ █ █▄▀ █▄█▄▄▄▀█▀▄  ▄▀▀█▀ ▄▄▄ █ ▀▀▀ █
▀▀▀▀▀▀▀ █▄▀▄█ ▀▄█▄▀ ▀▄█ █ █▄▀▄▀ ▀ ▀▀▀▀▀▀▀
██ █  ▀█    ███ ██  ▄█▄▀▄▀▀██ █▀▄▄▀▀▀ ▀▀▄
██ ▀ █▀█▄▀█▄██▀ ▀ █▀ ▄▄▄▀██ █▀▀▀█ █▀  █▀
█▀█▄  ▀█▀▀ ▄█▄█▄▀ █▀█  ██▀▄█▀ ▄██▀█ ▀   ▀
▀██   ▀▄▄▀▄ ██ ▀██▄▀▄█▀▀█▄▀▄▀█▀▄▄▀▀█▄▀ █▀
▀█▄▄ ▀▀▄ ▀▀▀▄▀ ██▄▄ ▀ ▀ █▀ ▀▄▀ █ █ █▀█▄ ▄
███ █ ▀██ ▀▄▀█ ▄▄▀ ▄ ▄█▄▄ ▀▄██▄ ▀▄█▀▀  ▀▀
▀█▄█▄ ▀ ▀▄▄█  ▀▄ █ ▄▀▀  ▄▄▀▄█▄█ ██▀▀▀▀▀ █
▀ ▀▀▄ ▀▄█ ▄  ██▄▀▀██▄▄█▄ ▄█▄   █▀ ▄ ▄▀██
█ ▀▄▄▄▀█▀██▄▄▀▄▄▀█▄ █ ▄██▀▄  ▄ ▄▄█▀█ █ ▄
▀ ██ ▀▀▀▄ ▀██▄▄▄ ██  █▀█▀▄█ ▄▄▀▄▀▀ ▄█ ▀█▀
▀▄▄█ █▀ █▄▀ ██▀█▄ █▄▀████▀▄██ ▄▄▀█ █▀█▀▀█
▀  ▄▄ ▀▄▄▄▄▀█ ▄▀▀█████  ▀▀▀▄ █ ▄█▀█▀▀█▀▀▀
▀ ▀▀  ▀▀▄▀▀▄▄▀█▀▄ ▄███ ▀██▀▀█ █▄█▀▀▀█▄ █
█▀▀▀▀▀█ ▀▄█▄▀ ▄  █ ▀▄▄▄▄▄█▀  ▀▀ █ ▀ █ ▄█▄
█ ███ █ ▄ █▄ ▄ █▄▄  ▀█  █ ▄ ▀██ █▀█▀█ ▀█▀
█ ▀▀▀ █ ▄▄▀█ █ ▄ ▄█▄ ▄▄▀█▄▀  ▄█ █ ▀ █▀ █▀
▀▀▀▀▀▀▀ ▀▀   ▀  ▀    ▀▀   ▀  ▀  ▀▀▀  ▀▀▀
alice@cold.machine:~$ 
```

### Wallet Options

```bash
usage: burrow wallet
 -h,--help                      display help message
 -p,--passphrase <PASSPHRASE>   optional seed passphrase
 -s,--sentence <SENTENCE>       mnemonic sentence
 -q,--qrcode                    optional seed passphrase
```